import type { APIEmbed, Interaction, MessageCreateOptions } from 'discord.js';
import { EmbedBuilder, InteractionResponse, Message, NewsChannel, TextChannel } from 'discord.js';
export type MessageBuilderFileResolvable = NonNullable<MessageCreateOptions['files']>[number];
export type MessageBuilderResolvable = Omit<MessageCreateOptions, 'embed' | 'disableMentions' | 'reply'> & {
    embeds?: MessageCreateOptions['embeds'];
    expiration?: number;
};
/**
 * A message builder class, it implements the {@link MessageCreateOptions} interface.
 */
export declare class MessageBuilder implements MessageCreateOptions {
    /**
     * Whether the message should be spoken aloud.
     * @default false
     */
    tts?: MessageCreateOptions['tts'];
    /**
     * The nonce for the message.
     * @default ''
     */
    nonce?: MessageCreateOptions['nonce'];
    /**
     * The content for the message. If set to undefined and the builder is used to edit, the content will not be
     * replaced.
     */
    content?: MessageCreateOptions['content'];
    /**
     * The embeds for the message. If set to undefined and the builder is used to edit, the embed will not be replaced.
     * @remark There is a maximum of 10 embeds in 1 message
     */
    embeds: MessageCreateOptions['embeds'];
    /**
     * Which mentions should be parsed from the message content.
     */
    allowedMentions?: MessageCreateOptions['allowedMentions'];
    /**
     * Files to send with the message. This should not be set when editing a message, as Discord does not support
     * editing file attachments.
     */
    files?: MessageCreateOptions['files'];
    /**
     * Expiration time for the message in milliseconds.
     * @param options
     */
    expiration?: number;
    /**
     * The default values for all MessageBuilder instances.
     */
    static defaults: MessageBuilderResolvable;
    constructor(options?: MessageBuilderResolvable);
    /**
     * Sets the value for the {@link MessageBuilder.tts} field.
     * @param tts Whether or not the message should be spoken aloud.
     */
    setTTS(tts?: boolean): this;
    /**
     * Sets the value for the {@link MessageBuilder.nonce} field.
     * @param nonce The nonce for the message.
     */
    setNonce(nonce?: string): this;
    /**
     * Sets the value for the {@link MessageBuilder.content} field.
     * @param content The content for the message. If set to undefined and the builder is used to edit, the content will
     * not be replaced.
     */
    setContent(content?: string): this;
    /**
     * Sets the value for the {@link MessageBuilder.embed} field.
     * @param embeds The embeds for the message. If set to undefined and the builder is used to edit, the embed will not be
     * replaced. There is a maximum of 10 embeds per message
     * @remark When providing more than 10 embeds, the array will automatically be sliced down to the first 10.
     */
    setEmbeds(embeds: EmbedBuilder | APIEmbed | MessageCreateOptions['embeds']): this;
    /**
     * Sets the value for the {@link MessageBuilder.allowedMentions} field.
     * @param allowedMentions Which mentions should be parsed from the message content.
     */
    setAllowedMentions(allowedMentions?: MessageCreateOptions['allowedMentions']): this;
    /**
     * Adds a new value for the {@link MessageBuilder.files} field array.
     * @param file The file to add to the {@link MessageBuilder.files} field array.
     */
    addFile(file: MessageBuilderFileResolvable): this;
    /**
     * Sets a single value for the {@link MessageBuilder.files} field array.
     * @param file The file to send with the message. This should not be set when editing a message, as Discord does not
     * support editing file attachments.
     */
    setFile(file: MessageBuilderFileResolvable): this;
    /**
     * Sets the value for the {@link MessageBuilder.files} field.
     * @param files The files to send with the message. This should not be set when editing a message, as Discord does
     * not support editing file attachments.
     */
    setFiles(files?: MessageBuilderFileResolvable[]): this;
    setExpiration(expiration?: number): this;
    followUp(interaction: Interaction | Message): Promise<InteractionResponse | Message | undefined>;
    send(channel: TextChannel | NewsChannel): Promise<Message>;
}
//# sourceMappingURL=MessageBuilder.d.ts.map