"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageBuilder = void 0;
const discord_js_1 = require("discord.js");
/**
 * A message builder class, it implements the {@link MessageCreateOptions} interface.
 */
class MessageBuilder {
    constructor(options) {
        var _a, _b, _c, _d, _e, _f, _g;
        this.tts = (_a = options === null || options === void 0 ? void 0 : options.tts) !== null && _a !== void 0 ? _a : MessageBuilder.defaults.tts;
        this.nonce = (_b = options === null || options === void 0 ? void 0 : options.nonce) !== null && _b !== void 0 ? _b : MessageBuilder.defaults.nonce;
        this.content = (_c = options === null || options === void 0 ? void 0 : options.content) !== null && _c !== void 0 ? _c : MessageBuilder.defaults.content;
        this.embeds = (_d = options === null || options === void 0 ? void 0 : options.embeds) !== null && _d !== void 0 ? _d : MessageBuilder.defaults.embeds;
        this.allowedMentions = (_e = options === null || options === void 0 ? void 0 : options.allowedMentions) !== null && _e !== void 0 ? _e : MessageBuilder.defaults.allowedMentions;
        this.files = (_f = options === null || options === void 0 ? void 0 : options.files) !== null && _f !== void 0 ? _f : MessageBuilder.defaults.files;
        this.expiration = (_g = options === null || options === void 0 ? void 0 : options.expiration) !== null && _g !== void 0 ? _g : MessageBuilder.defaults.expiration;
    }
    /**
     * Sets the value for the {@link MessageBuilder.tts} field.
     * @param tts Whether or not the message should be spoken aloud.
     */
    setTTS(tts) {
        this.tts = tts;
        return this;
    }
    /**
     * Sets the value for the {@link MessageBuilder.nonce} field.
     * @param nonce The nonce for the message.
     */
    setNonce(nonce) {
        this.nonce = nonce;
        return this;
    }
    /**
     * Sets the value for the {@link MessageBuilder.content} field.
     * @param content The content for the message. If set to undefined and the builder is used to edit, the content will
     * not be replaced.
     */
    setContent(content) {
        this.content = content;
        return this;
    }
    /**
     * Sets the value for the {@link MessageBuilder.embed} field.
     * @param embeds The embeds for the message. If set to undefined and the builder is used to edit, the embed will not be
     * replaced. There is a maximum of 10 embeds per message
     * @remark When providing more than 10 embeds, the array will automatically be sliced down to the first 10.
     */
    setEmbeds(embeds) {
        // Ensure no more than 10 embeds are ever se
        if (!embeds) {
            this.embeds = [];
            return this;
        }
        embeds = Array.isArray(embeds) ? embeds : [embeds];
        if (embeds && embeds.length > 10) {
            embeds = embeds.slice(0, 10);
        }
        this.embeds = embeds.map(embed => embed instanceof discord_js_1.EmbedBuilder ? embed.data : embed);
        return this;
    }
    /**
     * Sets the value for the {@link MessageBuilder.allowedMentions} field.
     * @param allowedMentions Which mentions should be parsed from the message content.
     */
    setAllowedMentions(allowedMentions) {
        this.allowedMentions = allowedMentions;
        return this;
    }
    /**
     * Adds a new value for the {@link MessageBuilder.files} field array.
     * @param file The file to add to the {@link MessageBuilder.files} field array.
     */
    addFile(file) {
        var _a, _b;
        this.files = (_b = (_a = this.files) === null || _a === void 0 ? void 0 : _a.concat(file)) !== null && _b !== void 0 ? _b : [file];
        return this;
    }
    /**
     * Sets a single value for the {@link MessageBuilder.files} field array.
     * @param file The file to send with the message. This should not be set when editing a message, as Discord does not
     * support editing file attachments.
     */
    setFile(file) {
        this.files = [file];
        return this;
    }
    /**
     * Sets the value for the {@link MessageBuilder.files} field.
     * @param files The files to send with the message. This should not be set when editing a message, as Discord does
     * not support editing file attachments.
     */
    setFiles(files) {
        this.files = files;
        return this;
    }
    setExpiration(expiration) {
        if (!expiration || expiration < 0) {
            this.expiration = undefined;
        }
        this.expiration = expiration;
        return this;
    }
    followUp(interaction) {
        return __awaiter(this, void 0, void 0, function* () {
            if (interaction instanceof discord_js_1.Message) {
                return interaction.reply(this);
            }
            if (interaction.isRepliable()) {
                return interaction.editReply(this);
            }
        });
    }
    send(channel) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.embeds) {
                // this.embeds = this.embeds.map(embed => assertEmbedSize(embed as APIEmbed));
            }
            const message = yield channel.send(this);
            if (this.expiration) {
                setTimeout(() => message.delete().catch(() => { }), this.expiration);
            }
            return message;
        });
    }
}
exports.MessageBuilder = MessageBuilder;
/**
 * The default values for all MessageBuilder instances.
 */
MessageBuilder.defaults = {
    expiration: 10000
};
